@@ -10,6 +10,7 @@ import zipfile
 import json
 import unittest
 import requests
+import inspect
 from selenium import webdriver
 from selenium.common.exceptions import NoSuchElementException, NoAlertPresentException, NoSuchWindowException, \
     TimeoutException, StaleElementReferenceException, ElementClickInterceptedException, \
@@ -63,22 +64,16 @@ class Selenium(Functions, Parameters):
     message_container = None
     message_error = None
     json_on_loaded = None
-    global_date = time.strftime(
-        Parameters.date_format)  # formato dd/mm/aaaa global_time = time.strftime(Parameters.time_format)  # formato 24 houras
-    lista_pasos = []
-    lista_descripcion_pasos = []
-    driver_port_status = False
+    global_date = time.strftime(Parameters.date_format)
+    steps = []
     chrome_services = ServiceChrome()
     firefox_services = ServiceFirefox()
     ie_services = ServiceIexplorer()
     edge_services = ServiceEdge()
-    pid = None
-    record = None
-
 
     # INICIALIZA LOS DRIVER Y LOS CONFIGURA
     def open_browser(self, url=None, browser=Parameters.browser, options_headless=Parameters.headless,
-                     download_path=None, record=None, privated=True):
+                     download_path=None, record=Parameters.record, privated=True):
 
         """
             Description:
@@ -94,12 +89,9 @@ class Selenium(Functions, Parameters):
                     -El directorio base
                     -El navegador utilizado
         """
-        #print(f"Directorio Base: {Parameters.current_path}")
+
         Functions.LoggingFeature(f"Directorio Base: {Parameters.current_path}").send_log()
-        Functions.LoggingFeature(f"{self.color_message('YELLOW', 'AGUARDANDO:')} Inicio del navegador "
-              f"'{self.color_message('BLUE', browser)}'.").send_log()
-        #print(f"{self.color_message('YELLOW', 'AGUARDANDO:')} Inicio del navegador "
-              #f"'{self.color_message('BLUE', browser)}'.")
+        Functions.LoggingFeature(f"AGUARDANDO: Inicio del navegador '{browser}'").send_log()
         options = None
         if browser == "CHROME":
             options = webdriver.ChromeOptions()
@@ -129,8 +121,6 @@ class Selenium(Functions, Parameters):
         options.add_argument("--disable-extensions")
         options.add_argument("--dns-prefetch-disable")
         options.add_argument("--verbose")
-        #options.add_argument("--disable-popup-blocking")
-        #options.add_argument("--proxy-server='direct://'")
         options.add_argument("--proxy-bypass-list=*")
         options.add_argument("--disable-gpu") if os.name == "nt" else None
         if Parameters.environment == "Linux":
@@ -141,73 +131,77 @@ class Selenium(Functions, Parameters):
                 Selenium.driver = webdriver.Chrome(service=self.chrome_services, options=options)
                 Selenium.initialize_browser(self, url)
                 Selenium.driver.maximize_window()
-
         if Parameters.environment == "Windows":
-            if Parameters.environment_configuration == "standalone" or Parameters.environment_configuration is None:
-                Selenium.set_mode_debug(False)
-
+            if Parameters.environment_configuration == "standalone":
+                Selenium.set_mode_debug(True)
             if options_headless or Parameters.environment_configuration == "server":
                 options.add_argument("--headless")
                 Selenium.set_exception_loggin(True)
                 Selenium.set_mode_debug(False)
                 Selenium.set_highlight(False)
-            Selenium.select_browser(self, browser, options)
-            Selenium.initialize_browser(self, url, record)
-            Selenium.driver.maximize_window()
-            return Selenium.driver
+        Selenium.select_browser(self, browser, options)
+        Selenium.initialize_browser(self, url, record)
+        Selenium.driver.maximize_window()
+        Selenium.update_steps(self, f"Abrir el navegador '{browser.lower()}' e ingresar a '{url}'.")
+        return Selenium.driver
 
     def initialize_browser(self, url, record=False):
+
         """
         Description:
             Inicia el navegador configurado y navega hacia la url.
         Args:
             url: Url destino.
         """
+
         Selenium.driver.implicitly_wait(10)
         try:
             Selenium.driver.get(url)
             Selenium.windows = {'Principal': Selenium.driver.window_handles[0]}
-
             if (record is None and Parameters.record) or (record is True):
                 Selenium.record = True
                 path_video = f"{os.path.join(Functions.path_outputs, Functions.test_case_name)}.mp4"
                 self.screen_recorder = ScreenRecord(driver=self.driver, file_name=path_video)
                 self.screen_recorder.record_screen()
-
-
         except WebDriverException:
-            Selenium.tear_down(self)
+            Selenium.close_browser(self)
             unittest.TestCase().fail(f"--WebDriverException--No se ha podido establecer una "
                                      f"conexión con el ambiente de pruebas {url}.")
 
+    def update_steps(self, description):
+
+        """
+            Description:
+                Guarda informacion sobre los pasos realizados durante la ejecución.
+            Args:
+                description: Descripcion de lo que se esta ejecutando.
+        """
+
+        Selenium.steps.append({
+            "ORDER": len(Selenium.steps) + 1,
+            "DESCRIPTION": description,
+            "SCREENSHOT": Selenium.driver.get_screenshot_as_png()
+        })
+
     def select_browser(self, browser, options):
 
         """
-
             Description:
-
                 Permite configurar el navegador que se utilizará en la prueba.
-
             Args:
-
                 browser: Nombre del navegador.
-
                 options: Argumentos opcionales del navegador.
-
         """
 
         try:
             if browser == "CHROME":
                 URL = "https://storage.googleapis.com/chrome-for-testing-public/124.0.6367.155/win64/chrome-win64.zip"
                 PATH_CHROME_TEMP = "chromium_windows_x64.temp"
-                if Parameters.environment_configuration == "server":
-                    options.binary_location =  r'C:\\Chromium\\chrome.exe'
-                else:
-                    if not os.path.exists("C:/ChromeForTesting/chrome-win64/chrome.exe"):
-                        Selenium.download_chromium(URL, PATH_CHROME_TEMP)
-                        Selenium.unzip_and_rename(PATH_CHROME_TEMP, "C:/ChromeForTesting/")
-                        os.remove(PATH_CHROME_TEMP)
-                    options.binary_location = "C:/ChromeForTesting/chrome-win64/chrome.exe"
+                if not os.path.exists("C:/ChromeForTesting/chrome-win64/chrome.exe"):
+                    Selenium.download_chromium(URL, PATH_CHROME_TEMP)
+                    Selenium.unzip_and_rename(PATH_CHROME_TEMP, "C:/ChromeForTesting/")
+                    os.remove(PATH_CHROME_TEMP)
+                options.binary_location = "C:/ChromeForTesting/chrome-win64/chrome.exe"
                 Selenium.driver = webdriver.Chrome(service=self.chrome_services, options=options)
 
             elif browser == "FIREFOX":
@@ -218,16 +212,23 @@ class Selenium(Functions, Parameters):
 
             elif browser == "EDGE":
                 Selenium.driver = webdriver.Edge(service=self.edge_services, options=options)
-            #Selenium.pid = Selenium.driver.service.process.pid
 
         except Exception as e:
             Functions.exception_logger(e)
-            Selenium.tear_down(self)
+            Selenium.close_browser(self)
             unittest.TestCase().skipTest(f"El web driver no esta disponible para esta prueba. {e}")
 
-
     @staticmethod
     def download_chromium(url, save_path):
+
+        """
+            Description:
+                Realiza la descarga de ChromeForTesting.
+            Args:
+                url: Link de descarga del navegador.
+                save_path: Ruta donde se dejara el archivo
+        """
+
         with requests.get(url, stream=True) as response:
             response.raise_for_status()
             with open(save_path, 'wb') as f:
@@ -236,33 +237,53 @@ class Selenium(Functions, Parameters):
 
     @staticmethod
     def unzip_and_rename(zip_file, destination_folder):
+
+        """
+            Description:
+                Descomprime el archivo de descarga.
+            Args:
+                zip_file: Ruta del archivo que debe descomprimirse.
+                destination_folder: Ruta donde debe dejarse el archivo descomprimido.
+        """
+
         with zipfile.ZipFile(zip_file, 'r') as zip_ref:
             zip_ref.extractall(destination_folder)
 
+    def close_browser(self):
+
+        """
+            Description:
+                Cierra el navegador y sus instancias.
+        """
+
+        if Selenium.driver is not None:
+            try:
+                Selenium.driver.close()
+                Selenium.driver.quit()
+                Selenium.driver = None
+            except Exception as e:
+                print(f"Ah ocurrido un error al intentar cerrar el navegador. {e}")
+
     def tear_down(self):
 
         """
-            Descripcion:
+            Description:
                 Finaliza la ejecución cerrando el Web Driver.
         """
-        Functions.create_grid_by_sources(self.data_cache, "Datos del cache")
         try:
             if Selenium.data_cache not in ([], {}):
+                Functions.create_grid_by_sources(self.data_cache, "Datos del cache")
                 print("====================Inicio Cache===================")
                 pprint.pprint(Selenium.data_cache)
                 print("=====================Fin Cache=====================")
-            Functions.LoggingFeature(f"{Selenium.color_message('YELLOW', 'AGUARDANDO:')} Se cerrará el web driver.").send_log()
-            #print(f"{Selenium.color_message('YELLOW', 'AGUARDANDO:')} Se cerrará el web driver.")
-            if Selenium.driver is not None:
-                Selenium.driver.close()
-                Selenium.driver.quit()
+            Functions.LoggingFeature(f"AGUARDANDO: Se cerrará el web driver.").send_log()
+            Selenium.close_browser(self)
 
         except Exception as e:
             Functions.exception_logger(e)
 
         finally:
-            #print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Finaliza la ejecución.")
-            Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Finaliza la ejecución.").send_log()
+            Functions.LoggingFeature(f"REALIZADO: Finaliza la ejecución.").send_log()
             if Selenium.record:
                 self.screen_recorder.stop_recording()
 
@@ -383,31 +404,28 @@ class Selenium(Functions, Parameters):
         elements = None
         try:
             elements = Selenium.driver.find_element(type_locator, indentify)
-            Functions.LoggingFeature(f"Se interactuo con el elemento {indentify}").send_log()
-            Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se detecto el elemento web "
-                  f"'{Selenium.color_message('BLUE', entity)}' utilizando el "
-                  f"identificador {type_locator} apuntando a '{indentify}'").send_log()
+            Functions.LoggingFeature(f"REALIZADO: Se detecto el elemento web '{entity}' utilizando el identificador "
+                                     f"{type_locator} apuntando a '{indentify}'").send_log()
             find_element = True
 
         except NoSuchElementException:
             Selenium.exception = "NoSuchElementException"
-            Functions.LoggingFeature(f"No se pudo encontrar el elemento web '{Selenium.color_message('BLUE', entity)}'"
-                  f" utilizando el identificador {type_locator} apuntando a '{indentify}'").send_log()
+            Functions.LoggingFeature(f"No se pudo encontrar el elemento web '{entity}' utilizando el identificador "
+                                     f"'{type_locator}' apuntando a '{indentify}'").send_log()
+
             Selenium.message_error = f"No se pudo encontrar el elemento web '{entity}' utilizando el identificador " \
                                      f"{type_locator} apuntando a '{indentify}'"
 
         except TimeoutException:
             Selenium.exception = "TimeoutException"
-            Functions.LoggingFeature(f"Se agoto el tiempo de busqueda intentando encontrar el elemento web "
-                  f"'{Selenium.color_message('BLUE', entity)}' utilizando el identificador"
-                  f" {type_locator} apuntando a '{indentify}'").send_log()
+            Functions.LoggingFeature(f"Se agoto el tiempo de busqueda intentando encontrar el elemento web '{entity}'"
+                                     f" utilizando el identificador '{type_locator}' apuntando a '{indentify}'").send_log()
+
             Selenium.message_error = f"Se agoto el tiempo de busqueda intentando encontrar el elemento web '{entity}'" \
-                                     f" utilizando el identificador " \
-                                     f"{type_locator} apuntando a '{indentify}'"
+                                     f" utilizando el identificador {type_locator} apuntando a '{indentify}'"
         except Exception as e:
+            Functions.LoggingFeature(f"ERROR: Ha ocurrido un error inesperado en tiempo de ejecución.").send_log()
             Functions.exception_logger(e)
-            Functions.LoggingFeature(f"Ha ocurrido un error inesperado en tiempo de ejecución.").send_log()
-        Selenium.lista_descripcion_pasos.append(indentify)
         return elements, find_element
 
     def highlight(self, element: WebElement):
@@ -489,21 +507,14 @@ class Selenium(Functions, Parameters):
             Selenium.wait(Parameters.time_between_retries)
 
         if not find_element:
-            if os.getenv('PYBOT_SYSTEM') is not None:
-                if os.getenv('PYBOT_SYSTEM').lower() != "server":
-                    self.image_for_debugger_report()
-                    self.steps_case = ""
-                    for i in range(len(self.lista_pasos)):
-                        self.steps_case += f"* {self.lista_pasos[i]}: {self.lista_descripcion_pasos[i]}\n"
-                    status_code_returned = Selenium.debugger(self, entity)
-                    if status_code_returned == 1:  # retry / refab
-                        Functions.value_to_find = Debugger.element_new_val_to_find
-                        # Releer el json
-                        Functions.get_json_file(self, Functions.json_on_loaded)
-                        Functions.set_retry(self, 3)
-                        return Functions.capture_element(self, entity)
-            Selenium.screenshot(self, "Ultima screenshot antes de finalizar la ejecución")
-            Selenium.tear_down(self)
+            if Parameters.environment_configuration != "server":
+                Selenium.update_steps(self, f"El objeto {entity} no se visualiza en pantalla.")
+                status_code_returned = Selenium.debugger(self, entity)
+                if status_code_returned == 1:  # retry / refab
+                    Selenium.set_retry(self, 3)
+                    return Selenium.capture_element(self, entity)
+            Selenium.screenshot(self, "Ultima screenshot antes de finalizar la ejecución.")
+            Selenium.close_browser(self)
             unittest.TestCase().fail(f"--{Selenium.exception}-- El objeto {entity} no se visualiza en pantalla. "
                                      f"REF:{Selenium.value_to_find}")
 
@@ -548,10 +559,17 @@ class Selenium(Functions, Parameters):
             "VALUE TO FIND": Selenium.value_to_find,
             "JSON PATH": Selenium.complete_json_path,
             "JSON STRING": Selenium.json_strings,
+            "STEPS": Selenium.steps,
+            "RESOURCE": Functions.data_resource,
+            "CACHE": Functions.data_cache,
             "CASE NAME": self.case_name
         }
+        if metadata["JSON PATH"] is None:
+            stack = inspect.stack()
+            metadata["FILE PATH"] = stack[3].filename
+
         returned_code = None
-        if Selenium.get_mode_execution() and not Selenium.get_mode_browser():
+        if Parameters.debug and not Parameters.headless:
             response = str(Debugger(metadata))
             returned_code = int(response.split("||")[0])
             Selenium.value_to_find = response.split("||")[1]
@@ -570,27 +588,21 @@ class Selenium(Functions, Parameters):
                 Si el archivo fue encontrado imprime "get_json_file: " + json_path",
                 en caso contrario imprime "get_json_file: No se encontro el Archivo " + file".
         """
+
         Selenium.json_on_loaded = file
         json_path = os.path.join(self.path_json, f"{file}.json")
         Selenium.complete_json_path = json_path
         try:
             with open(json_path, "r", encoding='utf8') as read_file:
                 Selenium.json_strings = json.loads(read_file.read())
-                Functions.LoggingFeature(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se a cargado el respositorio de objetos "
-                      f"'{Selenium.color_message('BLUE', f'{file}.json')}' encontrado en el directorio "
-                      f"'{json_path}'.").send_log()
-                #print(f"{Selenium.color_message('GREEN', 'REALIZADO:')} Se a cargado el respositorio de objetos "
-                      #f"'{Selenium.color_message('BLUE', f'{file}.json')}' encontrado en el directorio "
-                      #f"'{json_path}'.")
+                Functions.LoggingFeature(f"REALIZADO: Se a cargado el respositorio de objetos '{file}.json' "
+                                         f"encontrado en el directorio '{json_path}'.").send_log()
+
         except FileNotFoundError:
             Selenium.json_strings = False
-            Functions.LoggingFeature(
-                f"{Selenium.color_message('RED', 'ERROR:')} No se encontro "
-                  f"'{Selenium.color_message('BLUE', f'{file}.json')}' en el directorio '{json_path}'.").send_log()
-            #print(f"{Selenium.color_message('RED', 'ERROR:')} No se encontro "
-                  #f"'{Selenium.color_message('BLUE', f'{file}.json')}' en el directorio '{json_path}'.")
+            Functions.LoggingFeature(f"ERROR: No se encontro '{file}.json' en el directorio '{json_path}'.").send_log()
+            Selenium.close_browser(self)
             unittest.TestCase().skipTest(f"get_json_file: No se encontro el Archivo {file}")
-            Selenium.tear_down(self)
 
     def get_entity(self, entity):
 
@@ -615,7 +627,7 @@ class Selenium(Functions, Parameters):
                 Functions.exception_logger(e)
                 unittest.TestCase().skipTest(f"get_entity: No se encontro la key a la cual se hace referencia:"
                                              f"{entity}.")
-                Selenium.tear_down()
+                Selenium.close_browser(self)
 
             return True
 
@@ -784,7 +796,7 @@ class Selenium(Functions, Parameters):
             except Exception as e:
                 Functions.exception_logger(e)
                 print(f"No se pudo realizar la screenshot de pantalla.")
-            Selenium.tear_down(self)
+            Selenium.close_browser(self)
             unittest.TestCase().fail("--TimeoutException-- No se ha podido realizar la carga de la página.")
 
     def scroll_to(self, locator, y=None, x=None):
@@ -883,6 +895,7 @@ class Selenium(Functions, Parameters):
                     action.move_to_element(localizador)
                     action.click(localizador)
                     action.perform()
+                    Selenium.update_steps(self, f"Posicionar el puntero sobre '{locator}'.")
                     print(u"mouse_over: " + locator)
                     return True
 
@@ -892,6 +905,7 @@ class Selenium(Functions, Parameters):
                     action.move_to_element(localizador)
                     action.click(localizador)
                     action.perform()
+                    Selenium.update_steps(self, f"Posicionar el puntero sobre '{locator}'.")
                     print(u"mouse_over: " + locator)
                     return True
 
@@ -901,6 +915,7 @@ class Selenium(Functions, Parameters):
                     action.move_to_element(localizador)
                     action.click(localizador)
                     action.perform()
+                    Selenium.update_steps(self, f"Posicionar el puntero sobre '{locator}'.")
                     print(u"mouse_over: " + locator)
                     return True
 
@@ -910,17 +925,18 @@ class Selenium(Functions, Parameters):
                     action.move_to_element(localizador)
                     action.click(localizador)
                     action.perform()
+                    Selenium.update_steps(self, f"Posicionar el puntero sobre '{locator}'.")
                     print(u"mouse_over: " + locator)
                     return True
 
             except TimeoutException:
                 print(u"mouse_over: No presente " + locator)
-                Selenium.tear_down(self)
+                Selenium.close_browser(self)
                 return None
 
             except StaleElementReferenceException:
                 print(u"element " + locator + " is not attached to the DOM")
-                Selenium.tear_down(self)
+                Selenium.close_browser(self)
                 return None
 
     def double_click_element(self, element: WebElement):
@@ -936,6 +952,7 @@ class Selenium(Functions, Parameters):
         mouse_action.double_click(element)
         mouse_action.perform()
 
+
     def drag_and_drop(self, origin_object, target_object):
 
         """
@@ -1276,22 +1293,6 @@ class Selenium(Functions, Parameters):
             Functions.exception_logger(e)
             print(f"No se pudo realizar la screenshot de pantalla.")
 
-    def image_for_debugger_report(self):
-
-        """
-            Description:
-                Saca screenshot de pantalla para los reportes de allure y se agrega la descripción de la misma.
-            Returns:
-                Retorna la imágen y descripción de la screenshot de pantalla.
-        """
-        try:
-            base_folder = f"C:\\testing-Automation\\projects\\{str(self.project_name)}\\src\\outputs\\"
-            if "jira_report.png" not in base_folder:
-                self.memory_image = Selenium.driver.get_screenshot_as_png()
-        except Exception as e:
-            print(f"Hubo inconvenientes al intentar crear la carpeta contenedora de las 'report image'")
-
-    # SERVICIOS WEB ####################################################################################################
     def get_random(self, min_range, max_range):
 
         """
@@ -1438,6 +1439,7 @@ class Selenium(Functions, Parameters):
         return Functions.execute_sp_base_sqlserver(self, db_name, query, parameters)
 
     def get_list_base_sqlserver(self, db_name, query):
+
         """
             Description:
                 Realiza conexión y consulta a base de datos con la libreria pyodbc. El metodo incluye la
@@ -1841,18 +1843,17 @@ class Selenium(Functions, Parameters):
                 en caso contrario imprime por consola "VERIFICAR: No se pudo escribir el archivo."
                 y devuelve FALSE.
         """
+
         return Functions.write_cell(self, cell, value, name, folder, sheet)
 
     @staticmethod
     def available_port():
 
         """
-
             Description:
                 Busca un puerto disponible.
             Returns:
                 Devuelve el puerto disponible.
-
         """
 
         import socket
@@ -1864,37 +1865,40 @@ class Selenium(Functions, Parameters):
     def set_exception_loggin(value: bool):
 
         """
-
-        Description:
-            Configura el logeo de las excepciones
-
-        Args:
-            value: true o false
-
+            Description:
+                Configura el logeo de las excepciones
+            Args:
+                value: true o false
         """
+
         Functions.set_exception_loggin(value)
 
     @staticmethod
     def color_message(color, message):
-        """
-        Description: Colorea el string del color indicado de entre una lista de colores.
-
-        Args:
-            color: puede ser de color red, blue, yellow o green.
-            message: string a colorear.
-
-        Returns:
-            string coloreado.
 
         """
+            Description:
+                Colorea el string del color indicado de entre una lista de colores.
+            Args:
+                color: puede ser de color red, blue, yellow o green.
+                message: string a colorear.
+            Returns:
+                string coloreado.
+        """
+
         return Functions.color_message(color, message)
 
     def open_new_tab(self, web_url):
-        # Abrir nueva tab
+
+        """
+            Description:
+                Abre un nuevo tab en el navegador.
+            Args:
+                web_url: link de la web a la que se accedera en el nuevo tab.
+        """
+
         Selenium.driver.execute_script("window.open('about:blank', 'secondtab');")
-        # Cambio de prioridad a la segunda ventana
         Selenium.driver.switch_to.window("secondtab")
-        # ingresar a la web deseada en la nueva tab
         Selenium.driver.get(f'{web_url}')
 
     def check_environment_files(self, father_attribute, attribute_to_search, data_find, data_inner_key,
@@ -2086,7 +2090,7 @@ class ElementUI(Selenium):
                     self.exception = "StaleElementReferenceException"
                     self.message = f'--{self.exception}-- No se ha podido {self.message} ' \
                                    f'debido a que el objeto "{self.entity}" a sido actualizado repentinamente. ' \
-                                   f'REF:{self.json_ValueToFind }'
+                                   f'REF:{self.json_ValueToFind}'
                     Selenium.message_error = self.message
                     Selenium.exception = self.exception
 
@@ -2128,20 +2132,20 @@ class ElementUI(Selenium):
 
                 self.element = Selenium.capture_element(self, self.entity)
             else:
+                Selenium.update_steps(self, f"{self.message.capitalize()} sobre '{self.entity}'.")
                 if Selenium.debugger(self, self.element) == 1:
                     self.retry = 0
                     self.element = Selenium.capture_element(self, self.entity)
                 else:
                     Selenium.screenshot(self, "Ultima screenshot antes de finalizar la ejecución.")
-                    Selenium.tear_down(self)
+                    Selenium.close_browser(self)
                     if len(self.message) == 0:
                         self.message = f"--{self.exception}-- {self.message}"
                     unittest.TestCase().fail(self.message)
 
-        Functions.LoggingFeature(f"{Functions.color_message('GREEN', 'REALIZADO:')} Se pudo {self.message} sobre "
-              f"'{Functions.color_message('BLUE', self.entity)}'.").send_log()
+        Functions.LoggingFeature(f"REALIZADO: Se pudo {self.message} sobre '{self.entity}'.").send_log()
         Selenium.message_container = self.message
-        Selenium.lista_pasos.append(action)
+        Selenium.update_steps(self, f"{self.message.capitalize()} sobre '{self.entity}'.")
         if out is not None:
             return out
 
